(**************************************************************************)
(*                                                                        *)
(*  FORK                                                                  *)
(*                                                                        *)
(*  Author:  Franois Pottier, INRIA Paris-Rocquencourt                   *)
(*  Version: 20101109                                                     *)
(*                                                                        *)
(*  The copyright to this code is held by Institut National de Recherche  *)
(*  en Informatique et en Automatique (INRIA). All rights reserved. This  *)
(*  file is distributed under the license CeCILL-C (see file LICENSE).    *)
(*                                                                        *)
(**************************************************************************)

open Identifier

(* ------------------------------------------------------------------------- *)

(* Sorts. *)

(* We have several sorts of identifiers. Each sort defines an independent
   namespace. *)

let kind_sort : sort = (0, "kind")
let type_sort : sort = (1, "type")
let term_sort : sort = (2, "term")

(* ------------------------------------------------------------------------- *)

(* Kinds. *)

type kind =
  | KStar
  | KDefined of identifier
  | KArrow of kind * kind
  | KLater of kind

(* ------------------------------------------------------------------------- *)

(* Types. *)

type quantifier =
  | QForall
  | QExists

type ty =
  | TyVar of identifier
  | TyAbs of identifier * kind option * locty
  | TyApp of locty * locty
  | TyConstraint of locty * kind
  | TyUnit
  | TyArrow of Error.location * locty * locty
  | TyPair of Error.location * locty * locty
  | TyQuantified of quantifier * identifier * kind option * locty
  | TyRawQuantifier of quantifier * kind

and locty =
  Error.location * ty

(* ------------------------------------------------------------------------- *)

(* Statements. *)

type statement =
  | SEquation of locty * locty
  | SForall of identifier * kind option * statement

(* ------------------------------------------------------------------------- *)

(* Terms. *)

type term =
  | TeVar of identifier
  | TeAbs of identifier * locty * locterm
  | TeApp of locterm * locterm
  | TeTyAbs of identifier * kind option * locterm
  | TeTyApp of locterm * locty
  | TePack of locterm * locty * locty
  | TeUnpack of identifier * identifier * locterm * locterm
  | TeConstraint of locterm * locty
  | TeType of identifier * locty * locterm
  | TeLet of identifier * locterm * locterm
  | TeUnit
  | TePair of locterm * locterm
  | TeLetPair of identifier option * identifier option * locterm * locterm

and locterm =
    Error.location * term

(* ------------------------------------------------------------------------- *)

(* Declarations. *)

type declaration =
  | DKind of identifier * kind
  | DType of identifier * kind * locty
  | DDefaultKind of identifier * kind
  | DLemma of statement
  | DTerm of identifier * locty * locterm

type declarations =
    declaration list

type program =
    declarations

